<?php

namespace Drupal\module_status;

use Drupal\Core\Extension\ModuleHandlerInterface;

/**
 * Class RssIssueReaderService.
 */
class RssIssueReaderService implements RssIssueReaderServiceInterface {

  /**
   * @var \Drupal\Core\Extension\ModuleHandler
   */
  private $moduleHandler;

  /**
   * @var null|int
   */
  private $countOfAllIssues = NULL;

  /**
   * @var null|array
   */
  private $issues = NULL;

  /**
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $moduleHandler
   */
  public function __construct(ModuleHandlerInterface $moduleHandler) {
    $this->moduleHandler = $moduleHandler;
  }

  /**
   * @return array|null
   */
  public function getModuleIssues() {
    if ($this->issues === NULL) {
      $this->retrieveIssues();
    }
    return $this->issues;
  }

  /**
   * @return int|null
   */
  public function getCountOfAllIssues() {
    if ($this->countOfAllIssues === NULL) {
      $this->retrieveIssues();
    }
    return $this->countOfAllIssues;
  }

  private function retrieveIssues(): void {
    $moduleList = $this->moduleHandler->getModuleList();
    $moduleNames = [];
    foreach ($moduleList as $moduleNameKey => $module) {
      if (substr($module->getPathname(), 0, 4) === "core") {
        continue;
      }
      $moduleNames[] = $moduleNameKey;
    }

    $this->issues = [];

    foreach ($moduleNames as $module) {
      $currentIssues = $this->getCriticalIssues($module);
      if ($currentIssues === FALSE) {
        continue;
      }
      $countIssues = count($currentIssues);
      $this->issues[$module]['count'] = $countIssues;
      $this->issues[$module]['link'] = $this->getLinkToIssuesByModuleName($module);
      $this->countOfAllIssues += $countIssues;
    }
  }

  /**
   * @param string $moduleName
   *
   * @return mixed
   */
  private function getCriticalIssues(string $moduleName) {
    $rssUrl = $this->getRssUrlFromModuleName($moduleName);

    /** todo:use a better way to request a website gulp or curl */
    $rssUrl = @file_get_contents($rssUrl);
    if (!$rssUrl) {
      \Drupal::logger('module_status')->notice(
        "Unable to fetch issues of the @module module.",
        ['@module' => $moduleName]
      );
      return FALSE;
    }
    $simpleXMLElement = new \SimpleXMLElement(trim($rssUrl));
    return $simpleXMLElement->channel->item;
  }

  /**
   * @param $moduleName
   *
   * @return string
   */
  private function getRssUrlFromModuleName($moduleName) {
    return 'https://www.drupal.org/project/issues/rss/' . $moduleName . '?priorities=400&categories=1&version=8.x';
  }

  /**
   * @param $moduleName
   *
   * @return string
   */
  private function getLinkToIssuesByModuleName($moduleName) {
    return 'https://www.drupal.org/project/issues/' . $moduleName . '?priorities=400&categories=1&version=8.x';
  }

}
